#include "libdnf.hxx"
#include <array>
#include <cstddef>
#include <cstdint>
#include <cstring>
#include <exception>
#include <new>
#include <string>
#include <type_traits>
#include <utility>

namespace rust {
inline namespace cxxbridge1 {
// #include "rust/cxx.h"

struct unsafe_bitcopy_t;

#ifndef CXXBRIDGE1_RUST_STRING
#define CXXBRIDGE1_RUST_STRING
class String final {
public:
  String() noexcept;
  String(const String &) noexcept;
  String(String &&) noexcept;
  ~String() noexcept;

  String(const std::string &);
  String(const char *);
  String(const char *, std::size_t);
  String(const char16_t *);
  String(const char16_t *, std::size_t);

  String &operator=(const String &) &noexcept;
  String &operator=(String &&) &noexcept;

  explicit operator std::string() const;

  const char *data() const noexcept;
  std::size_t size() const noexcept;
  std::size_t length() const noexcept;
  bool empty() const noexcept;

  const char *c_str() noexcept;

  std::size_t capacity() const noexcept;
  void reserve(size_t new_cap) noexcept;

  using iterator = char *;
  iterator begin() noexcept;
  iterator end() noexcept;

  using const_iterator = const char *;
  const_iterator begin() const noexcept;
  const_iterator end() const noexcept;
  const_iterator cbegin() const noexcept;
  const_iterator cend() const noexcept;

  bool operator==(const String &) const noexcept;
  bool operator!=(const String &) const noexcept;
  bool operator<(const String &) const noexcept;
  bool operator<=(const String &) const noexcept;
  bool operator>(const String &) const noexcept;
  bool operator>=(const String &) const noexcept;

  void swap(String &) noexcept;

  String(unsafe_bitcopy_t, const String &) noexcept;

private:
  friend void swap(String &lhs, String &rhs) noexcept { lhs.swap(rhs); }

  std::array<std::uintptr_t, 3> repr;
};
#endif // CXXBRIDGE1_RUST_STRING

#ifndef CXXBRIDGE1_RELOCATABLE
#define CXXBRIDGE1_RELOCATABLE
namespace detail {
template <typename... Ts>
struct make_void {
  using type = void;
};

template <typename... Ts>
using void_t = typename make_void<Ts...>::type;

template <typename Void, template <typename...> class, typename...>
struct detect : std::false_type {};
template <template <typename...> class T, typename... A>
struct detect<void_t<T<A...>>, T, A...> : std::true_type {};

template <template <typename...> class T, typename... A>
using is_detected = detect<void, T, A...>;

template <typename T>
using detect_IsRelocatable = typename T::IsRelocatable;

template <typename T>
struct get_IsRelocatable
    : std::is_same<typename T::IsRelocatable, std::true_type> {};
} // namespace detail

template <typename T>
struct IsRelocatable
    : std::conditional<
          detail::is_detected<detail::detect_IsRelocatable, T>::value,
          detail::get_IsRelocatable<T>,
          std::integral_constant<
              bool, std::is_trivially_move_constructible<T>::value &&
                        std::is_trivially_destructible<T>::value>>::type {};
#endif // CXXBRIDGE1_RELOCATABLE

namespace {
namespace repr {
struct PtrLen final {
  void *ptr;
  ::std::size_t len;
};
} // namespace repr
} // namespace
} // namespace cxxbridge1

namespace behavior {
class missing {};
missing trycatch(...);

template <typename Try, typename Fail>
static typename ::std::enable_if<
    ::std::is_same<decltype(trycatch(::std::declval<Try>(), ::std::declval<Fail>())),
                 missing>::value>::type
trycatch(Try &&func, Fail &&fail) noexcept try {
  func();
} catch (const ::std::exception &e) {
  fail(e.what());
}
} // namespace behavior
} // namespace rust

extern "C" {
const char *cxxbridge1$exception(const char *, ::std::size_t);
} // extern "C"

static_assert(
    ::rust::IsRelocatable<::dnfcxx::DnfPackage>::value,
    "type dnfcxx::DnfPackage should be trivially move constructible and trivially destructible in C++ to be used as a non-pinned mutable reference in signature of `dnf_package_get_nevra`, `dnf_package_get_name`, `dnf_package_get_evr` in Rust");
static_assert(
    ::rust::IsRelocatable<::dnfcxx::DnfRepo>::value,
    "type dnfcxx::DnfRepo should be trivially move constructible and trivially destructible in C++ to be used as a non-pinned mutable reference in signature of `dnf_repo_get_id`, `dnf_repo_get_timestamp_generated` in Rust");

namespace dnfcxx {
extern "C" {
::rust::repr::PtrLen dnfcxx$cxxbridge1$dnf_package_get_nevra(::dnfcxx::DnfPackage &pkg, ::rust::String *return$) noexcept {
  ::rust::String (*dnf_package_get_nevra$)(::dnfcxx::DnfPackage &) = ::dnfcxx::dnf_package_get_nevra;
  ::rust::repr::PtrLen throw$;
  ::rust::behavior::trycatch(
      [&] {
        new (return$) ::rust::String(dnf_package_get_nevra$(pkg));
        throw$.ptr = nullptr;
      },
      [&](const char *catch$) noexcept {
        throw$.len = ::std::strlen(catch$);
        throw$.ptr = const_cast<char *>(::cxxbridge1$exception(catch$, throw$.len));
      });
  return throw$;
}

::rust::repr::PtrLen dnfcxx$cxxbridge1$dnf_package_get_name(::dnfcxx::DnfPackage &pkg, ::rust::String *return$) noexcept {
  ::rust::String (*dnf_package_get_name$)(::dnfcxx::DnfPackage &) = ::dnfcxx::dnf_package_get_name;
  ::rust::repr::PtrLen throw$;
  ::rust::behavior::trycatch(
      [&] {
        new (return$) ::rust::String(dnf_package_get_name$(pkg));
        throw$.ptr = nullptr;
      },
      [&](const char *catch$) noexcept {
        throw$.len = ::std::strlen(catch$);
        throw$.ptr = const_cast<char *>(::cxxbridge1$exception(catch$, throw$.len));
      });
  return throw$;
}

::rust::repr::PtrLen dnfcxx$cxxbridge1$dnf_package_get_evr(::dnfcxx::DnfPackage &pkg, ::rust::String *return$) noexcept {
  ::rust::String (*dnf_package_get_evr$)(::dnfcxx::DnfPackage &) = ::dnfcxx::dnf_package_get_evr;
  ::rust::repr::PtrLen throw$;
  ::rust::behavior::trycatch(
      [&] {
        new (return$) ::rust::String(dnf_package_get_evr$(pkg));
        throw$.ptr = nullptr;
      },
      [&](const char *catch$) noexcept {
        throw$.len = ::std::strlen(catch$);
        throw$.ptr = const_cast<char *>(::cxxbridge1$exception(catch$, throw$.len));
      });
  return throw$;
}

::rust::repr::PtrLen dnfcxx$cxxbridge1$dnf_package_get_arch(::dnfcxx::DnfPackage &pkg, ::rust::String *return$) noexcept {
  ::rust::String (*dnf_package_get_arch$)(::dnfcxx::DnfPackage &) = ::dnfcxx::dnf_package_get_arch;
  ::rust::repr::PtrLen throw$;
  ::rust::behavior::trycatch(
      [&] {
        new (return$) ::rust::String(dnf_package_get_arch$(pkg));
        throw$.ptr = nullptr;
      },
      [&](const char *catch$) noexcept {
        throw$.len = ::std::strlen(catch$);
        throw$.ptr = const_cast<char *>(::cxxbridge1$exception(catch$, throw$.len));
      });
  return throw$;
}

::rust::repr::PtrLen dnfcxx$cxxbridge1$dnf_repo_get_id(::dnfcxx::DnfRepo &repo, ::rust::String *return$) noexcept {
  ::rust::String (*dnf_repo_get_id$)(::dnfcxx::DnfRepo &) = ::dnfcxx::dnf_repo_get_id;
  ::rust::repr::PtrLen throw$;
  ::rust::behavior::trycatch(
      [&] {
        new (return$) ::rust::String(dnf_repo_get_id$(repo));
        throw$.ptr = nullptr;
      },
      [&](const char *catch$) noexcept {
        throw$.len = ::std::strlen(catch$);
        throw$.ptr = const_cast<char *>(::cxxbridge1$exception(catch$, throw$.len));
      });
  return throw$;
}

::rust::repr::PtrLen dnfcxx$cxxbridge1$dnf_repo_get_timestamp_generated(::dnfcxx::DnfRepo &repo, ::std::uint64_t *return$) noexcept {
  ::std::uint64_t (*dnf_repo_get_timestamp_generated$)(::dnfcxx::DnfRepo &) = ::dnfcxx::dnf_repo_get_timestamp_generated;
  ::rust::repr::PtrLen throw$;
  ::rust::behavior::trycatch(
      [&] {
        new (return$) ::std::uint64_t(dnf_repo_get_timestamp_generated$(repo));
        throw$.ptr = nullptr;
      },
      [&](const char *catch$) noexcept {
        throw$.len = ::std::strlen(catch$);
        throw$.ptr = const_cast<char *>(::cxxbridge1$exception(catch$, throw$.len));
      });
  return throw$;
}
} // extern "C"
} // namespace dnfcxx
